@@ -43,10 +43,11 @@
 #' and choose an approximate algorithm with an acceptable level of 
 #' approximation to make the problem tractable.
 #'
-#' **Note on method grid:** The grid method implements a version of Point
+#' **Note on method grid:** The finite grid method implements a version of Point
 #' Based Value Iteration (PBVI). The used belief points are by default created
 #' using points that are reachable from the initial belief (`start`) by
-#' following all combinations of actions and observations. The size of the grid
+#' following all combinations of actions and observations. The size of the grid is
+#' by default 10,000 and 
 #' can be set via `parameter = list(fg_points = 100)`. Alternatively,
 #' different strategies can be chosen using the parameter `fg_type`. In
 #' this implementation, the user can also specify manually a grid of belief
@@ -359,6 +360,9 @@ solve_POMDP <- function(model,
   if (is.character(model))
     model <- read_POMDP(model)
   
+  if (!inherits(model, "POMDP"))
+    stop("x needs to be a POMDP!") 
+  
   if (is.null(horizon))
     horizon <- model$horizon
   if (is.null(horizon))
@@ -660,7 +664,7 @@ solve_POMDP <- function(model,
   model
 }
 
-
+#' @export
 print.POMDP_solution <- function(x, ...) {
   cat("POMDP solution\n\n")
   print(unclass(x))
